<?php
/**
 * Provides easier access to meta information about Nacoma's object types
 */
class help_nacoma_Model
{
	/**
	 * This tells the user what they need to know to correctly pass in arguments
	 *
	 * @param $type string
	 * @return string
	 */
	static function translate_internal_property_type_to_external($type) {
		return str_replace(
			array(
				'mselect',
				'select',
				'checkbox',
				'hidden',
				'textarea',
				'image',
				'bool',
			),
			array(
				'array',
				'string',
				'array',
				'string',
				'string',
				'string',
				'boolean',
			),
			$type
		);
	}

	/**
	 * One might wonder why these methods are in help_nacoma_Model and not http_api_nacoma_Model.
	 * You have my empathy.
	 *
	 * @param $object_type string e.g. 'host'
	 * @param $extra_sections array
	 * @throws Exception when given invalid object type
	 * @return string
	 */
	function get_html_help_for_object_type($object_type, array $extra_sections = array())
	{
		$available_objects = $this->get_available_objects(true);
		$property_descriptions = json_decode(file_get_contents(__DIR__.'/../data/helptexts.json'), true);

		if(!array_key_exists($object_type, $available_objects)) {
			throw new Exception("No help found for '$object_type' since it is an invalid object type");
		}
		$response = array_merge(
			array(
				'description' => $available_objects[$object_type]['doc'],
				'menu' => array()
			),
			$extra_sections
		);
		$output = null;

		$parameter_list = "<h2>Properties of <strong>$object_type</strong> resources</h2>
			<table>
			<tr>
				<th>Property (<span class='required'>required</span>)</th>
				<th>Type</th>
				<th>Default</th>
				<th>Description</th>
			</tr>";
		$required_rows = array();
		$non_required_rows = array();
		foreach($available_objects[$object_type]['properties']['var_index'] as $property => $type) {
			if($type === 'norender') {
				// Hide property from help text.. It's
				// deprecated, internal, or just hidden,
				// beacuse, go fish
				continue;
			}
			$type = self::translate_internal_property_type_to_external($type);
			$property_row = "<td><var>$property</var></td>
				<td>$type</td>
				<td>";
			if(isset($available_objects[$object_type]['properties']['default'][$property]) &&
				$available_objects[$object_type]['properties']['default'][$property]) {
				if(is_array($available_objects[$object_type]['properties']['default'][$property])) {
					$property_row .= "[".implode(', ', $available_objects[$object_type]['properties']['default'][$property])."]";
				} else {
					$property_row .= $available_objects[$object_type]['properties']['default'][$property];
				}
			}
			$property_row .= "</td>
				<td>";
			if(is_array($property_descriptions[$property]) && !isset($property_descriptions[$property][$object_type])) {
				bug("No property '$property' description for object type '$object_type' found");
			}
			$property_row .= (is_array($property_descriptions[$property]) ? $property_descriptions[$property][$object_type] : $property_descriptions[$property])."</td>
			</tr>";
			if(in_array($property, $available_objects[$object_type]['properties']['required'])) {
				$property_row = "<tr class='required'>".$property_row;
				$required_rows[$property] = $property_row;
			} else {
				$property_row = "<tr>".$property_row;
				$non_required_rows[$property] = $property_row;
			}
		}

		ksort($required_rows);
		$parameter_list .= implode("\n", $required_rows);
		ksort($non_required_rows);
		$parameter_list .= implode("\n", $non_required_rows);
		$example_rows = $required_rows ? array_keys($required_rows) : array(
			'example_property_1',
			'example_property_2',
			'example_property_3'
		);
		$output .= $parameter_list.'</table>';

		// @todo this will NOT work for Monitor installations
		// accessed behind proxies which modify the URI
		//
		// @todo adapt /api to /a-possible/hinderance/api
		$example_xml_wget_uri = 'http'.(isset($_SERVER['HTTPS']) && 'on' == $_SERVER['HTTPS'] ? 's' : null).'://'.(isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : 'localhost').'/api';
		$example_xml_wget_uri .= "/config/$object_type";
		$example_json = array();
		foreach($example_rows as $required_row) {
			$example_json[] = '"'.$required_row.'": "..."';
		}
		$example_json = "{".implode(', ', $example_json)."}";
		$footer = "
<h2>Live data</h2>
<p>List real objects from your Monitor installation in <a href='./../../config/$object_type?format=xml'>XML</a>, <a href='./../../config/$object_type?format=json'>JSON</a> or <a href='./../../config/$object_type?format=html'>HTML</a>.</p>
<p>If you would like to view a single object and its properties, edit the URI to point at <em>/api/config/$object_type/&lt;name of object here&gt;?format=xml</em>.</p>

<p>Remember that <a href='/api/help/config#persistent_changes'>all changes needs to be additionally confirmed before they are used live</a>.</p>

<h2>Example usage</h2>
<p>Follow the examples below to extract $object_type objects remotely.</p>

<p>A friendly reminder:</p>
<dl>
	<dt>GET</dt>
	<dd>Lists object</dd>

	<dt>POST</dt>
	<dd>Creates object</dd>

	<dt>PATCH</dt>
	<dd>Updates object</dd>

	<dt>PUT</dt>
	<dd>Replaces object</dd>

	<dt>DELETE</dt>
	<dd>Removes object</dd>
</dl>

<h3 id='example_curl'>cURL in terminal</h3>
<div class='curl'>
<h4>GET</h4>
<pre><code>curl '$example_xml_wget_uri?format=xml' -u '&lt;your username&gt;:&lt;your password&gt;'</code></pre>

<h4>POST</h4>
<pre><code>curl -H 'content-type: application/json' -d '$example_json' '$example_xml_wget_uri' -u '&lt;your username&gt;:&lt;your password&gt;'</code></pre>
<h5>Via a file</h5>
<pre><code>curl -H 'content-type: application/json' -d '@&lt;filename of local json&gt;.json' '$example_xml_wget_uri' -u '&lt;your username&gt;:&lt;your password&gt;'</code></pre>

<h4>PATCH</h4>
<pre><code>curl -X PATCH -H 'content-type: application/json' -d '{\"a_property\": \"a_value\"}' -u '&lt;your username&gt;:&lt;your password&gt;' '$example_xml_wget_uri/&lt;object to update&gt;'</code></pre>

<h4>PUT</h4>
<pre><code>curl -H 'content-type: application/xml' '$example_xml_wget_uri/&lt;object to overwrite&gt;?format=xml' -T '&lt;your-xml-file.xml&gt;' -u '&lt;your username&gt;:&lt;your password&gt;'</code></pre>

<h4>DELETE</h4>
<pre><code>curl -X DELETE '$example_xml_wget_uri/&lt;object to delete&gt;' -u '&lt;your username&gt;:&lt;your password&gt;'</code></pre>
</div>

<h3 id='example_python'>Python</h3>
<div class='python'>
<h4>GET</h4>
<pre><code>import requests
r = requests.get('$example_xml_wget_uri?format=json', auth=('&lt;your username&gt;', '&lt;your password&gt;'))
print r.text</code></pre>

<h4>POST</h4>
<pre><code>import requests
import json
data = {\n\t";
		$footer .= implode(",\n\t",
			array_map(function($r) {
				return "\"$r\": \"...\"";
			},
			$example_rows
		));
$footer .= "\n}
r = requests.post('$example_xml_wget_uri', data=json.dumps(data), auth=('&lt;your username&gt;', '&lt;your password&gt;'), headers={'content-type': 'application/json'})
print r.text</code></pre>

<h4>PATCH</h4>
<pre><code>import requests
import json
data = {
	\"a_property\": \"a_value\"
}
r = requests.patch('$example_xml_wget_uri/&lt;object to update&gt;', data=json.dumps(data), auth=('&lt;your username&gt;', '&lt;your password&gt;'), headers={'content-type': 'application/json'})
print r.text</code></pre>

<h4>PUT</h4>
<pre><code>import requests
data = \"\"\"&lt;root&gt;&lt;a_property_here&gt;a value&lt;/a_property_here&gt;&lt;another_property&gt;a value&lt;/another_property&gt;&lt;/root&gt;\"\"\"
r = requests.put('$example_xml_wget_uri/&lt;object to overwrite&gt;', data=data, auth=('&lt;your username&gt;', '&lt;your password&gt;'), headers={'content-type': 'application/xml'})
print r.text</code></pre>

<h4>DELETE</h4>
<pre><code>import requests
r = requests.delete('$example_xml_wget_uri/&lt;object to delete&gt;', auth=('&lt;your username&gt;', '&lt;your password&gt;'))
print r.text</code></pre>
</div>

<h3 id='example_php'>PHP with cURL</h3>
<div class='php'>
<h4>GET</h4>
<pre><code>\$a_handle = curl_init('$example_xml_wget_uri?format=json');
curl_setopt(\$a_handle, CURLOPT_USERPWD, '&lt;your username&gt;:&lt;your password&gt;');
curl_setopt(\$a_handle, CURLOPT_RETURNTRANSFER, TRUE);
\${$object_type}s = curl_exec(\$a_handle);
echo \${$object_type}s;</code></pre>

<h4>POST</h4>
<pre><code>\$data = json_encode(array(\n\t";
	$footer .= implode(",\n\t",
		array_map(function($r) {
			return "\"$r\" => \"...\"";
		},
		$example_rows
	));
$footer .= "
));
\$a_handle = curl_init('$example_xml_wget_uri');
curl_setopt(\$a_handle, CURLOPT_USERPWD, '&lt;your username&gt;:&lt;your password&gt;');
curl_setopt(\$a_handle, CURLOPT_RETURNTRANSFER, TRUE);
curl_setopt(\$a_handle, CURLOPT_POSTFIELDS, \$data);
curl_setopt(\$a_handle, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
curl_setopt(\$a_handle, CURLOPT_SSL_VERIFYPEER, false);
\${$object_type}s = curl_exec(\$a_handle);</code></pre>

<h4>PATCH</h4>
<pre><code>\$data = json_encode(array(
	\"a_property\" => \"a_value\"
));
\$a_handle = curl_init('$example_xml_wget_uri/&lt;object to update&gt;');
curl_setopt(\$a_handle, CURLOPT_USERPWD, '&lt;your username&gt;:&lt;your password&gt;');
curl_setopt(\$a_handle, CURLOPT_RETURNTRANSFER, TRUE);
curl_setopt(\$a_handle, CURLOPT_POSTFIELDS, \$data);
curl_setopt(\$a_handle, CURLOPT_CUSTOMREQUEST, 'PATCH');
curl_setopt(\$a_handle, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
curl_setopt(\$a_handle, CURLOPT_SSL_VERIFYPEER, false);
\${$object_type}s = curl_exec(\$a_handle);</code></pre>

<h4>PUT</h4>
<pre><code>\$data = json_encode(array(\n\t";
	$footer .= implode(",\n\t",
		array_map(function($r) {
			return "\"$r\" => \"...\"";
		},
		$example_rows
	));
$footer .= "
));
\$a_handle = curl_init('$example_xml_wget_uri/&lt;object to overwrite&gt;');
curl_setopt(\$a_handle, CURLOPT_USERPWD, '&lt;your username&gt;:&lt;your password&gt;');
curl_setopt(\$a_handle, CURLOPT_RETURNTRANSFER, TRUE);
curl_setopt(\$a_handle, CURLOPT_CUSTOMREQUEST, 'PUT');
curl_setopt(\$a_handle, CURLOPT_POSTFIELDS, \$data);
curl_setopt(\$a_handle, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
curl_setopt(\$a_handle, CURLOPT_SSL_VERIFYPEER, false);
\${$object_type}s = curl_exec(\$a_handle);</code></pre>

<h4>DELETE</h4>
<pre><code>\$a_handle = curl_init('$example_xml_wget_uri/&lt;object to delete&gt;');
curl_setopt(\$a_handle, CURLOPT_USERPWD, '&lt;your username&gt;:&lt;your password&gt;');
curl_setopt(\$a_handle, CURLOPT_RETURNTRANSFER, TRUE);
curl_setopt(\$a_handle, CURLOPT_CUSTOMREQUEST, 'DELETE');
curl_setopt(\$a_handle, CURLOPT_SSL_VERIFYPEER, false);
\$result = curl_exec(\$a_handle);</code></pre>
</div>

<p><a href='./'>Back to general API help.</a></p>";
		$output .= $footer;

		$response['body' ] = $output;
		$response['menu' ] = $this->list_objects();

		return $response;
	}

	/**
	 * @param $base_uri string = null This will be prepended to the href
	 * @return array [[title] => string, [items] => [string] => [resource] => string]
	 */
	function list_objects($base_uri = null) {
		$available_objects = $this->get_available_objects(true);
		$property_descriptions = json_decode(file_get_contents(__DIR__.'/../data/helptexts.json'), true);
		$items = array();
		foreach($available_objects as $object_name => $properties) {
			$items[ucfirst(str_replace('_', ' ', $object_name))] = array(
				'resource' => $base_uri.urlencode($object_name)
			);
		}

		return array(
			'title' => 'Object types',
			'items' => $items
		);
	}

	/**
	 * @param $detailed boolean = false
	 * @return array e.g. [host] => host_nacoma_Model
	 */
	function get_available_objects($detailed = false) {
		$manifest = Module_Manifest_Model::get('http_api_exposed_classes');
		if(!$detailed) {
			return $manifest;
		}
		$available_objects = array();
		foreach($manifest as $pretty_name => $class_name) {
			$rc = new ReflectionClass($class_name);
			$doc_comment = $rc->getDocComment();
			$doc_comment = preg_replace("/(^[\\s]*\\/\\*\\*)
				|(^[\\s]\\*\\/)
				|(^[\\s]*\\*?)
				|(^[\\s|\\t]*)/ixm", "", $doc_comment);

			$doc_comment = str_replace("\r", "", $doc_comment);
			$doc_comment = preg_replace("/([\\t])+/", "\t", $doc_comment);
			$doc_comment = trim($doc_comment);
			$doc_comment = preg_replace('~(https?://[\S]+)~', '<a href="$1" target="_blank">$1</a>', $doc_comment);
			$doc_comment = preg_replace('~\\n\\n~', "<br /><br />", $doc_comment);

			$available_objects[$pretty_name] = array_merge(
				get_object_vars(new $class_name),
				array(
					'doc' => $doc_comment
				)
			);
		}

		return $available_objects;
	}
}
